//
//  ImageViewViewController.swift
//  iOSAgentExample
//  Copyright © 2021 IBM Corp. All rights reserved.
//

import UIKit
import Combine
import InstanaAgent

class ImageViewViewController: UIViewController {

    lazy var imageView = UIImageView()
    lazy var session = { URLSession(configuration: URLSessionConfiguration.default) }()
    private var publisher: AnyCancellable?

    override func viewDidLoad() {
        super.viewDidLoad()

        view.addSubview(imageView)
        imageView.contentMode = .scaleAspectFill
        imageView.frame = view.bounds
        imageView.autoresizingMask = [.flexibleWidth, .flexibleHeight]
        imageView.isUserInteractionEnabled = true
        imageView.addGestureRecognizer(UITapGestureRecognizer(target: self, action: #selector(downloadImage)))

        downloadImage()
    }

    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        Instana.setView(name: "ImageView")
        Instana.reportEvent(name: "Custom Event", duration: 1001, backendTracingID: nil, error: NSError(domain: "Domain", code: 0, userInfo: nil), meta: ["Key": "Val"], viewName:"Image")
    }

    @objc func downloadImage() {
        let url = URL(string: "https://picsum.photos/900")!
        publisher = session.dataTaskPublisher(for: url)
            .receive(on: RunLoop.main)
            .map { UIImage(data: $0.data) }
            .replaceError(with: UIImage())
            .assign(to: \.image, on: self.imageView)
    }
}
